/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************/

package org.x4x.core.xmms2;

import java.util.Observer;

import org.x4x.core.KeepStateThread;
import org.x4x.core.ManagedThread;
import org.x4x.core.Management;
import org.x4x.core.ReturnValue;
import org.x4x.core.ThreadFactory;
import org.x4x.core.UpdateUIType;
import org.x4x.statics.StaticValues;
import org.xmms2.xmms2bindings.*;


/**
 * Xmms2 Threadfactory. You get one KeepStateThread, millions of ControlThread and almost the same 
 * #MedialibThreads ;). For description of reused methods have a look at core.ThreadFactory
 */
public final class Xmms2ThreadFactory extends ThreadFactory{
	private SWIGTYPE_p_xmmsc_connection_St controlConnection;
	private SWIGTYPE_p_xmmsc_connection_St medialibConnection;
	private SWIGTYPE_p_xmmsc_connection_St callbackConnection;
	private Xmms2KeepStateThread kst;
	private boolean spinDown = false;
	protected static String mySource = "client/X4X";
	protected static String[] sourcePrefence = new String[]{mySource, "client/*", "clients/*",
		"plugin/*", "plugins/*", "server"};
	protected static Boolean lockControlConnection = Boolean.FALSE;
	protected static Boolean lockMedialibConnection = Boolean.FALSE;
	
	/**
	 * Creates a new Xmms2ThreadFactory which means: load the library and init the connections to the daemon 
	 * 
	 * @param mm 
	 * @throws Exception 
	 *
	 */
	public Xmms2ThreadFactory(Management mm) throws Exception{
		super(mm);
		try {
			System.loadLibrary("xmms2java");
		} catch (UnsatisfiedLinkError e){
			logger.fatal(StaticValues.getLangString("clientlibUnavailable"));
			logger.debug(e.getMessage(), e);
			throw new Exception(StaticValues.getLangString("clientlibUnavailable"));
		} catch (Throwable t){
			logger.debug(t.getMessage(), t);
		}
		
		controlConnection = Xmmsclient.xmmsc_init("X4X-Control");
		medialibConnection = Xmmsclient.xmmsc_init("X4X-Medialib");
		callbackConnection = Xmmsclient.xmmsc_init("X4X-Callbacks");
		connect();
	}
	
	public ManagedThread getControlThread(ReturnValue ret, Observer obs) {
		if (!connected) connect();
		if (connected) return new Xmms2ControlThread(ret, obs, mm, kst, controlConnection);
		return null;
	}

	public ManagedThread getMedialibThread(ReturnValue ret, Observer obs) {
		if (!connected) connect();
		if (connected) return new Xmms2MedialibThread(ret, obs, mm, kst, medialibConnection);
		return null;
	}
	
	public KeepStateThread getKeepStateThread(Observer obs){
		if(kst == null)
			kst = new Xmms2KeepStateThread(this, obs, mm, callbackConnection);
		return kst;
	}
	
	public void spinDown() {
		logger.info("Spinning down");
		spinDown = true;
		if (connected){
			Xmmsclient.xmmsc_disconnect_callback_set(controlConnection, null, 0);
			Xmmsclient.xmmsc_disconnect_callback_set(callbackConnection, null, 0);
		}
		kst.spinDown();
		
		Xmmsclient.xmmsc_unref(controlConnection);
		Xmmsclient.xmmsc_unref(callbackConnection);
	}
	
	/**
	 * Connects a specific connection to xmms2d
	 * @param conn
	 */
	private void connect(SWIGTYPE_p_xmmsc_connection_St conn){
		if (!spinDown){
			String path = null;
			if (System.getProperty("java.specification.version").equals("1.5") &&
					System.getenv("XMMS_PATH") != null &&
					!System.getenv("XMMS_PATH").equals(""))
				path = System.getenv("XMMS_PATH");
			else if (System.getProperty("XMMS_PATH") != null && !System.getProperty("XMMS_PATH").equals(""))
				path = System.getProperty("XMMS_PATH");
			else if (!StaticValues.getSetting("serverip").equals("") && !StaticValues.getSetting("serverport").equals(""))
				path = "tcp://" + StaticValues.getSetting("serverip") + ":" + StaticValues.getSetting("serverport");
			if (path != null && (path.equals("") || path.equals("${env.XMMS_PATH}"))) path = null;
			if (Xmmsclient.xmmsc_connect(conn, path) <= 0){
				logger.warn(StaticValues.getLangString("xmms2dUnavailable"));
				connected = false;
			}
			else {
				connected = true;
				Xmmsclient.xmmsc_disconnect_callback_set(conn, XmmsclientConstants.DISCONNECT_CALLBACK, 0);
			}
		}
	}
	
	public void connect(){
		connect(controlConnection);
		connect(medialibConnection);
		connect(callbackConnection);
		if (connected)
			mm.restartBackend();
		else {
			setChanged();
			notifyObservers(new UpdateUIType(UpdateUIType.BACKEND_LOST, new Object[]{}));
		}
	}

	public long getKeepTime() {
		return 15000;
	}
}
