/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.lang.reflect.Field;
import java.util.Iterator;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XControl;
import org.Xwidgets.XFlatButton;
import org.Xwidgets.XPanel;
import org.apache.log4j.Logger;
import org.jdom.DataConversionException;
import org.jdom.Element;
import org.x4x.statics.StaticValues;


/**
 * That class simply represents <u>one</u>, and only one ButtonBar. It is, as every other
 * "ui-part class" in that package a singleton. The ButtonBar is dynamically built from
 * a jdom-Element and the Buttons are instantiated using some Reflection. You usually want
 * to use the Field me for usage in another XPanel or XFrame
 */

public class ButtonBar {
	private static ButtonBar instance = null;
	protected XPanel me;
	private Logger logger;
	protected XFlatButton play = null;
	protected XFlatButton stop = null;
	protected XFlatButton pause = null;
	protected XFlatButton next = null;
	protected XFlatButton prev = null;
	protected XFlatButton about = null;
	protected XFlatButton lists = null;
	private int size = 0;
	protected int lastIndex = 0;
	protected boolean packed = false;
	
	/**
	 * Create ButtonBar Object. used only once cause it's singleton
	 * 
	 * @param parent		parent Element (maybe dummy)
	 * @param bar			jdom Element
	 */
	private ButtonBar(XPanel parent, Element bar){
		logger = Logger.getLogger(getClass());
		try {
			buildMe(parent, bar);
		}catch (Exception e){
			logger.fatal(StaticValues.getLangString("uiError"));
			logger.debug(e.getMessage(), e);
			
		}
	}

	/**
	 * The method returns a singleton ButtonBar
	 * 
	 * @param parent		the ButtonBar's parentpanel
	 * @param bar			the jdom Element represent the bar
	 * @return				returns a singleton Object of a ButtonBar
	 */
	
	protected static ButtonBar getButtonBar(XPanel parent, Element bar){
		if (instance == null)
			instance = new ButtonBar(parent, bar);
		return instance;
	}
	
	/**
	 * Called by Constructor to build everything
	 * 
	 * @param parent
	 * @param bar
	 */
	private void buildMe(XPanel parent, Element bar){
		WidgetFactory wt = MainGUI.mainGui.myGUIFactory;
		String tmp = bar.getAttributeValue("size");
		if (tmp.equals("small")) size = 24;
		else if (tmp.equals("normal")) size = 40;
		else if (tmp.equals("medium")) size = 56;
		else if (tmp.equals("big")) size = 72;
		
		//if (bar.getAttributeValue("direction").equals("horizontal"))
			me = wt.getPanel(parent, Styles.SIMPLE);
			me.setLayoutData(MainGUI.mainGui.buttons);
		//else panel = wt.getPanel(parent, size+2, parent.getHeight(), Styles.BORDER);
		//int last = 0;
			
		for ( Iterator it = bar.getChildren().iterator(); it.hasNext(); ){
			Element child = (Element)it.next();
			if (!child.getName().endsWith("Button"))logger.info(StaticValues.getLangString("unknownElement"));
			else {
				String fieldname = "";
				try {
					fieldname = child.getName().toLowerCase().replaceAll("button", "");
					Field but = getClass().getDeclaredField(fieldname);
					but.set(this, getButton(wt, child, me, bar.getAttributeValue("direction")));
				} catch (SecurityException e) {
					logger.debug(e.getMessage(), e);
				} catch (NoSuchFieldException e) {
					logger.info(StaticValues.getLangString("ReflectionFieldException") + " \"" + 
							fieldname + "\": " + StaticValues.getLangString("noSuchField"));
					logger.debug(e.getMessage(), e);
				} catch (IllegalArgumentException e) {
					logger.debug(e.getMessage(), e);
				} catch (IllegalAccessException e) {
					logger.info(StaticValues.getLangString("ReflectionMethodException") + " \"" + 
							fieldname + "\": " + StaticValues.getLangString("noAccess"));
					logger.debug(e.getMessage(), e);
				} 
			}
		}
		
		try {
			if (bar.getAttribute("packed").getBooleanValue() == true){
				packed = true;
				me.pack();
			}
		} catch (DataConversionException e) {
			logger.debug(e.getMessage(), e);
		}
	}

	/**
	 * Builds a button on demand. called from buildMe()
	 * 
	 * @param wt
	 * @param button
	 * @param panel
	 * @param direction
	 * @return				built XButton
	 */
	private XFlatButton getButton(WidgetFactory wt, Element button, XPanel panel, String direction) {
		XFlatButton b = null;
		try {
			b = wt.getFlatButton(panel, Styles.SIMPLE);
			b.setText(StaticValues.getLangString(button.getName()));
			
			b.setImage(getImageForButton(button, size));
			b.setToolTipText(StaticValues.getLangString(button.getName()));
			//if (direction.equals("horizontal"))
			//else
				//b.setLocation(0, size*(index));
			b.addMouseListener(MouseListener.getInstance());
			fitAdd(b, 0);
		}catch (Exception e) {
			logger.debug(e.getMessage(), e);
		}
		return b;
	}
	
	protected void fitAdd(XControl b, int offset){
		b.setSize(size, size);
		b.setLocation(size*(lastIndex++)+offset, 0);
	}
	
	/**
	 * Get Image for Button
	 * 
	 * @param button
	 * @param size
	 * @return			String to right Image
	 */
	private String getImageForButton(Element button, int size) {
		String prefix = MainGUI.mainGui.iconPrefix + button.getName().toLowerCase().replaceAll("button", "");
		switch (size){
			case 24:	return prefix + "16x16" + ".png";
			case 40:	return prefix + "32x32" + ".png";
			case 56:	return prefix + "48x48" + ".png";
			case 72:	return prefix + "64x64" + ".png";
			default:	return "";
		}
	}
	
	public XPanel getPanel(){
		return me;
	}
}
