/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.plugins;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;

import org.Xwidgets.WidgetFactory;
import org.apache.log4j.Logger;
import org.x4x.gui.PluginConnector;

import sun.misc.Service;

public class PluginController implements Observer{
	private HashMap allPlugins;
	private HashMap runningPlugins;
	private HashMap activePlugins;
	private ClassLoader loader;
	private Logger logger;
	private List listeners;
	//private Thread.UncaughtExceptionHandler pluginExceptionHandler;
	
	public PluginController(URL pluginLocations[], WidgetFactory wf, PluginConnector pc){
		allPlugins = new HashMap();
		runningPlugins = new HashMap();
		activePlugins = new HashMap();
		listeners = new ArrayList();
		//pluginExceptionHandler = new PluginExceptionHandler();
		logger = Logger.getLogger(getClass());
		try {
			loader = new URLClassLoader(pluginLocations, Thread.currentThread().getContextClassLoader());
			Iterator it = Service.providers(Plugin.class, loader);
			while (it.hasNext()){
				try {
					Plugin pl = (Plugin)it.next();
					if (isValid(pl)){
						pl.wf = wf;
						allPlugins.put(pl.getName(), pl);
						if (pl instanceof UIPlugin)
							((UIPlugin)pl).pc = pc;
						if (pl instanceof VisualPlugin)
							((VisualPlugin)pl).pc = pc;
					}
				} catch (Exception e){
					e.printStackTrace();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void addPluginListener(PluginListener listener){
		listeners.add(listener);
	}
	
	public void removePluginListener(PluginListener listener){
		listeners.remove(listener);
	}
	
	public List spinDown(){
		ArrayList actives = new ArrayList();
		for (Iterator it = runningPlugins.keySet().iterator(); it.hasNext(); ){
			String name = ""+it.next();
			try {
				stopNoDeletePlugin((Plugin)runningPlugins.get(name));
			} catch (X4XPluginException e) {
				logger.debug(e.getMessage(), e);
			}
		}
		runningPlugins.clear();
		for (Iterator it = activePlugins.keySet().iterator(); it.hasNext(); ){
			String name = ""+it.next();
			try {
				actives.add(name);
				((Plugin)activePlugins.get(name)).destroy();
			} catch (X4XPluginException e) {
				logger.debug(e.getMessage(), e);
			}
		}
		activePlugins.clear();
		return actives;
	}
	
	public String[] getPluginNames(){
		String[] names = new String[allPlugins.size()];
		int i = 0;
		for (Iterator it = allPlugins.keySet().iterator(); it.hasNext();){
			names[i++] = ""+it.next();
		}
		return names;
	}
	
	public String getDescription(String name){
		if (allPlugins.containsKey(name))
			return ((Plugin)allPlugins.get(name)).getDescription();
		return "";
	}
	
	public Class getPluginclass(String name) throws X4XPluginException{
		if (!allPlugins.containsKey(name)) throw new X4XPluginException("No such plugin: " + name);
		Plugin pl = (Plugin)allPlugins.get(name);
		if (pl instanceof OutputPlugin) return OutputPlugin.class;
		else if (pl instanceof VisualPlugin) return VisualPlugin.class;
		else if (pl instanceof UIPlugin) return UIPlugin.class;
		return Object.class;
	}
	
	public void activatePlugin(String name, Properties configuration) throws X4XPluginException{
		if (!allPlugins.containsKey(name)) throw new X4XPluginException("No such plugin: " + name);
		Plugin pl = null;
		if (!activePlugins.containsKey(name)){
			try {
				pl = (Plugin)allPlugins.get(name);
				pl.init(configuration);
				activePlugins.put(pl.getName(), pl);
				for (Iterator i = listeners.iterator(); i.hasNext();){
					PluginListener p = (PluginListener)i.next();
					if (p != null)
						p.pluginActivated(name);
					else i.remove();
				}
			} catch (Exception e){
				notifyOfError(name, e);
				throw new X4XPluginException(e.getMessage());
			}
		}
	}
	
	public Plugin getActivePlugin(String name) throws X4XPluginException{
		if (!allPlugins.containsKey(name)) throw new X4XPluginException("No such plugin: " + name);
		return (Plugin)activePlugins.get(name);
	}
	
	public Plugin deactivatePlugin(String name) throws X4XPluginException{
		if (!allPlugins.containsKey(name)) throw new X4XPluginException("No such plugin: " + name);
		if (!activePlugins.containsKey(name)) throw new X4XPluginException("Plugin not active: " + name);
		try {
			activePlugins.remove(name);
			((Plugin)allPlugins.get(name)).destroy();
			for (Iterator i = listeners.iterator(); i.hasNext();){
				PluginListener p = (PluginListener)i.next();
				if (p != null)
					p.pluginDeactivated(name);
				else i.remove();
			}
		} catch (Throwable t){
			notifyOfError(name, t);
			throw new X4XPluginException(t.getMessage());
		}
		return (Plugin)allPlugins.get(name);
	}
	
	public void runPlugin(Plugin run){
		runningPlugins.put(run.getName(), run);
		run.addObserver(this);
		logger.debug("Starting plugin " + run.getName());
		Thread t = new Thread(run);
		run.setThread(t);
		//t.setUncaughtExceptionHandler(pluginExceptionHandler);
		t.start();
		for (Iterator i = listeners.iterator(); i.hasNext();){
			PluginListener p = (PluginListener)i.next();
			if (p != null)
				p.pluginStarted(run.getName());
			else i.remove();
		}
	}
	
	public void stopPlugin(Plugin run) throws X4XPluginException{
		try {
			stopNoDeletePlugin(run);
		} finally {
			runningPlugins.remove(run);
		}
	}
	
	private void stopNoDeletePlugin(Plugin run) throws X4XPluginException{
		try {
			run.stop();
			for (Iterator i = listeners.iterator(); i.hasNext();){
				PluginListener p = (PluginListener)i.next();
				if (p != null)
					p.pluginStopped(run.getName());
			}
		} catch (Throwable t){
			notifyOfError(run.getName(), t);
			throw new X4XPluginException(t.getMessage());
		}
	}

	private void notifyOfError(String name, Throwable t) {
		for (Iterator i = listeners.iterator(); i.hasNext();){
			PluginListener p = (PluginListener)i.next();
			if (p != null)
				p.errorOccured(name, t);
			else i.remove();
		}
	}

	private boolean isValid(Object o) {
		return (o instanceof Plugin && o.getClass().getPackage().equals(getClass().getPackage()));
	}
	
	public String[] getPlugins(Class cl){
		return new String[]{};
	}
	
	public boolean isActive(String name){
		return activePlugins.containsKey(name);
	}

	public boolean isRunning(String name){
		return runningPlugins.containsKey(name);
	}
	
	public void update(Observable arg0, Object arg1) {
		if (arg0 instanceof Plugin){
			Plugin pl = (Plugin)arg0;
			logger.debug("Plugin " + pl.getName() + " returned");
			if (!arg1.equals("")){
				notifyOfError(pl.getName(), new Exception(""+arg1));
				logger.error("Plugin " + pl.getName() + " reported error: " + arg1);
			}
			else {
				for (Iterator i = listeners.iterator(); i.hasNext();){
					PluginListener p = (PluginListener)i.next();
					if (p != null)
						p.pluginStopped(pl.getName());
					else i.remove();
				}
				runningPlugins.remove(pl.getName());
			}
		}
	}
	
	// TODO when we once switch to java5 uncomment the following
	/*private class PluginExceptionHandler implements Thread.UncaughtExceptionHandler{
		public void uncaughtException(Thread arg0, Throwable arg1) {
			logger.debug("Got uncaugt Exception from Thread " + arg0);
			logger.debug(arg1.getMessage(), arg1);
		}
	}*/
}
